@@ -36,7 +36,7 @@ class Config(object):
     Handle .env file format used by Foreman.
     """
     _BOOLEANS = {'1': True, 'yes': True, 'true': True, 'on': True,
-                 '0': False, 'no': False, 'false': False, 'off': False}
+                 '0': False, 'no': False, 'false': False, 'off': False, '': False}
 
     def __init__(self, repository):
         self.repository = repository
@@ -51,20 +51,28 @@ class Config(object):
 
         return self._BOOLEANS[value.lower()]
 
+    @staticmethod
+    def _cast_do_nothing(value):
+        return value
+
     def get(self, option, default=undefined, cast=undefined):
         """
         Return the value for option or default if defined.
         """
-        if option in self.repository:
-            value = self.repository.get(option)
+
+        # We can't avoid __contains__ because value may be empty.
+        if option in os.environ:
+            value = os.environ[option]
+        elif option in self.repository:
+            value = self.repository[option]
         else:
+            if isinstance(default, Undefined):
+                raise UndefinedValueError('{} not found. Declare it as envvar or define a default value.'.format(option))
+
             value = default
 
-        if isinstance(value, Undefined):
-            raise UndefinedValueError('%s option not found and default value was not defined.' % option)
-
         if isinstance(cast, Undefined):
-            cast = lambda v: v  # nop
+            cast = self._cast_do_nothing
         elif cast is bool:
             cast = self._cast_boolean
 
@@ -77,18 +85,18 @@ class Config(object):
         return self.get(*args, **kwargs)
 
 
-class RepositoryBase(object):
-    def __init__(self, source):
-        raise NotImplementedError
+class RepositoryEmpty(object):
+    def __init__(self, source=''):
+        pass
 
     def __contains__(self, key):
-        raise NotImplementedError
+        return False
 
-    def get(self, key):
-        raise NotImplementedError
+    def __getitem__(self, key):
+        return None
 
 
-class RepositoryIni(RepositoryBase):
+class RepositoryIni(RepositoryEmpty):
     """
     Retrieves option keys from .ini files.
     """
@@ -96,64 +104,59 @@ class RepositoryIni(RepositoryBase):
 
     def __init__(self, source):
         self.parser = ConfigParser()
-        self.parser.readfp(open(source))
+        with open(source) as file_:
+            self.parser.readfp(file_)
 
     def __contains__(self, key):
         return (key in os.environ or
                 self.parser.has_option(self.SECTION, key))
 
-    def get(self, key):
-        return (os.environ.get(key) or
-                self.parser.get(self.SECTION, key))
+    def __getitem__(self, key):
+        return self.parser.get(self.SECTION, key)
 
 
-class RepositoryEnv(RepositoryBase):
+class RepositoryEnv(RepositoryEmpty):
     """
     Retrieves option keys from .env files with fall back to os.environ.
     """
     def __init__(self, source):
         self.data = {}
 
-        for line in open(source):
-            line = line.strip()
-            if not line or line.startswith('#') or '=' not in line:
-                continue
-            k, v = line.split('=', 1)
-            k = k.strip()
-            v = v.strip().strip('\'"')
-            self.data[k] = v
+        with open(source) as file_:
+            for line in file_:
+                line = line.strip()
+                if not line or line.startswith('#') or '=' not in line:
+                    continue
+                k, v = line.split('=', 1)
+                k = k.strip()
+                v = v.strip().strip('\'"')
+                self.data[k] = v
 
     def __contains__(self, key):
         return key in os.environ or key in self.data
 
-    def get(self, key):
-        return os.environ.get(key) or self.data[key]
-
-
-class RepositoryShell(RepositoryBase):
-    """
-    Retrieves option keys from os.environ.
-    """
-    def __init__(self, source=None):
-        pass
-
-    def __contains__(self, key):
-        return key in os.environ
-
-    def get(self, key):
-        return os.environ[key]
+    def __getitem__(self, key):
+        return self.data[key]
 
 
 class AutoConfig(object):
     """
     Autodetects the config file and type.
+
+    Parameters
+    ----------
+    search_path : str, optional
+        Initial search path. If empty, the default search path is the
+        caller's path.
+
     """
     SUPPORTED = {
         'settings.ini': RepositoryIni,
         '.env': RepositoryEnv,
     }
 
-    def __init__(self):
+    def __init__(self, search_path=None):
+        self.search_path = search_path
         self.config = None
 
     def _find_file(self, path):
@@ -174,13 +177,10 @@ class AutoConfig(object):
     def _load(self, path):
         # Avoid unintended permission errors
         try:
-            filename = self._find_file(path)
+            filename = self._find_file(os.path.abspath(path))
         except Exception:
             filename = ''
-        Repository = self.SUPPORTED.get(os.path.basename(filename))
-
-        if not Repository:
-            Repository = RepositoryShell
+        Repository = self.SUPPORTED.get(os.path.basename(filename), RepositoryEmpty)
 
         self.config = Config(Repository(filename))
 
@@ -192,7 +192,7 @@ class AutoConfig(object):
 
     def __call__(self, *args, **kwargs):
         if not self.config:
-            self._load(self._caller_path())
+            self._load(self.search_path or self._caller_path())
 
         return self.config(*args, **kwargs)
 
@@ -209,16 +209,18 @@ class Csv(object):
     Produces a csv parser that return a list of transformed elements.
     """
 
-    def __init__(self, cast=text_type, delimiter=',', strip=string.whitespace):
+    def __init__(self, cast=text_type, delimiter=',', strip=string.whitespace, post_process=list):
         """
         Parameters:
         cast -- callable that transforms the item just before it's added to the list.
         delimiter -- string of delimiters chars passed to shlex.
         strip -- string of non-relevant characters to be passed to str.strip after the split.
+        tuple_ -- boolean to check if it is to return in tuple format.
         """
         self.cast = cast
         self.delimiter = delimiter
         self.strip = strip
+        self.post_process = post_process
 
     def __call__(self, value):
         """The actual transformation"""
@@ -228,5 +230,4 @@ class Csv(object):
         splitter.whitespace = self.delimiter
         splitter.whitespace_split = True
 
-        return [transform(s) for s in splitter]
-
+        return self.post_process(transform(s) for s in splitter)
